from django.contrib.admin.models import CHANGE
from django.utils.translation import gettext as _

from .modelform import ModelFormMixin
from .object import ObjectMixin


class ObjectFormMixin(ObjectMixin, ModelFormMixin):
    """Custom form view mixin on an object."""
    log_action_flag = CHANGE

    def get_form_valid_message(self):
        return _(
            '%s %s: {}' % (_(self.view_label), self.model_verbose_name)
        ).format(self.object).capitalize()



"""
A settings file to import boilerplate from.

.. py:data:: AUTHENTICATION_BACKENDS

    Contains the default django.contrib.auth.backends.ModelBackend and
    also crudlfap_auth.backends.ViewBackend which will introspect the
    view's authenticate and allowed_groups variables.

.. py:data:: CRUDLFAP_VIEWS

    List of default views to provide to Routers that were not spawned with any
    view.

.. py:data:: INSTALLED_APPS

    That list contains both :py:data:`CRUDLFAP_APPS` and :py:data:`DJANGO_APPS`
    and you can use them as such on a new project:

    .. code-block:: python

        from crudlfap.settings import INSTALLED_APPS

        INSTALLED_APPS = ['yourapp'] + INSTALLED_APPS

.. py:data:: CRUDLFAP_APPS

    List of apps CRUDLFA+ depends on, you can use it as such:

    .. code-block:: python

        from crudlfap.settings import CRUDLFAP_APPS

        INSTALLED_APPS = [
            'yourapp',
            'django.contrib.staticfiles',
            # etc
        ] + CRUDLFAP_APPS

.. py:data:: DJANGO_APPS

    This list contains all contrib apps from the Django project that CRUDLFA+
    should depend on. You can use it as such:

    .. code-block:: python

        from crudlfap.settings import CRUDLFAP_APPS, DJANGO_APPS

        INSTALLED_APPS = ['yourapp'] + CRUDLFAP_APPS + DJANGO_APPS

.. py:data:: TEMPLATES

    This list contains both :py:data:`DEFAULT_TEMPLATE_BACKEND` and
    :py:data:`CRUDLFAP_TEMPLATE_BACKEND` and works out of the box on an empty
    project. You can add it to your settings file by just importing it:

    .. code-block:: python

        from crudlfap.settings import TEMPLATES

.. py:data:: CRUDLFAP_TEMPLATE_BACKEND

   Configuration for Jinja2 and environment expected by
   CRUDLFA+ default templates. Add it to your own TEMPLATES setting using
   import:

    .. code-block:: python

       from crudlfap.settings import CRUDLFAP_TEMPLATE_BACKEND

       TEMPLATES = [
           # YOUR_BACKEND
           CRUDLFAP_TEMPLATE_BACKEND,
       ]

.. py:data:: DEFAULT_TEMPLATE_BACKEND

    Configuration for Django template backend with all builtin context
    processors. You can use it to define only your third backend as such:

    .. code-block:: python

        from crudlfap.settings import (
            CRUDLFAP_TEMPLATE_BACKEND,
            DEFAULT_TEMPLATE_BACKEND,
        )

        TEMPLATES = [
           # YOUR_BACKEND
           CRUDLFAP_TEMPLATE_BACKEND,
           DEFAULT_TEMPLATE_BACKEND,
        ]

.. py:data:: DEBUG

    Evaluate ``DEBUG`` env var as boolean, False by default.

.. py:data:: SECRET_KEY

    Get ``SECRET_KEY`` env var, or be ``'notsecret'`` by default.

    .. danger:: Raises an Exception if it finds both SECRET_KEY=notsecret and
                DEBUG=False.

.. py:data:: ALLOWED_HOSTS

    Split ``ALLOWED_HOSTS`` env var with commas, or be ``['*']`` by default.

    .. danger:: Raises an Exception if it finds both ALLOWED_HOSTS to be
                ``'*'`` and DEBUG=False.

.. py:data:: MIDDLEWARE

    A default MIDDLEWARE configuration you can import.

.. py:data:: OPTIONAL_APPS

    from crudlfap.settings import *
    # [...] your settings
    install_optional(OPTIONAL_APPS, INSTALLED_APPS)
    install_optional(OPTIONAL_MIDDLEWARE, MIDDLEWARE)

"""


import os
import sys
from pathlib import Path

from crudlfap.conf import install_optional  # noqa

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.dirname(
    os.path.abspath(__file__))))

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('SECRET_KEY', 'notsecret')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = bool(os.environ.get('DEBUG', False))

if 'HOST' in os.environ:
    ALLOWED_HOSTS = ['www.' + os.environ.get('HOST'), os.environ.get('HOST')]
elif DEBUG and 'ALLOWED_HOSTS' not in os.environ:
    ALLOWED_HOSTS = ['*']
else:
    ALLOWED_HOSTS = os.environ.get('ALLOWED_HOSTS', '').split(',')

CRUDLFAP_VIEWS = [
    'crudlfap.views.generic.CreateView',
    'crudlfap.views.generic.DeleteView',
    'crudlfap.views.generic.UpdateView',
    'crudlfap.views.generic.DetailView',
    'crudlfap.views.generic.ListView',
]

CRUDLFAP_APPS = [
    'crudlfap',
    'crudlfap_sites',
    'django_filters',
    'django_tables2',
    'crudlfap_auth',
    'ryzom',
    'py2js',
    'ryzom_django',
    'ryzom_django_mdc',
]

DJANGO_APPS = [
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'django.contrib.auth',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.sites',
    'django.contrib.staticfiles',
]

INSTALLED_APPS = DJANGO_APPS + CRUDLFAP_APPS

# CRUDLFA+ optional dependencies
OPTIONAL_APPS = [
    {'debug_toolbar': {'after': 'django.contrib.staticfiles'}},
    {'django_extensions': {'before': 'crudlfap'}},
]

MIDDLEWARE = [
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'ryzom_unpoly.middleware.UnpolyMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
]

OPTIONAL_MIDDLEWARE = [
    {'debug_toolbar.middleware.DebugToolbarMiddleware': None}
]

INTERNAL_IPS = ('127.0.0.1',)

CRUDLFAP_TEMPLATE_BACKEND = {
    'BACKEND': 'ryzom_django.template_backend.Ryzom',
    'NAME': 'ryzom',
}

DEFAULT_TEMPLATE_BACKEND = {
    'BACKEND': 'django.template.backends.django.DjangoTemplates',
    'DIRS': [],
    'APP_DIRS': True,
    'OPTIONS': {
        'context_processors': [
            'django.template.context_processors.debug',
            'django.template.context_processors.request',
            'django.contrib.auth.context_processors.auth',
            'django.contrib.messages.context_processors.messages',
        ],
    },
}

TEMPLATES = [CRUDLFAP_TEMPLATE_BACKEND, DEFAULT_TEMPLATE_BACKEND]
LOGIN_REDIRECT_URL = '/'
LOCALE_PATHS = (
    os.path.join(BASE_DIR, 'locale'),
)

WSGI_APPLICATION = 'crudlfap_example.wsgi.application'

AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
    'crudlfap_auth.backends.ViewBackend',
]


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': os.getenv('DB_ENGINE', 'django.db.backends.sqlite3'),
        'NAME': os.getenv('DB_NAME', 'db.sqlite3'),
        'HOST': os.getenv('DB_HOST'),
        'USER': os.getenv('DB_USER'),
        'PASSWORD': os.getenv('DB_PASS'),
    }
}

EMAIL_HOST = os.getenv('EMAIL_HOST', None)
EMAIL_PORT = os.getenv('EMAIL_PORT', None)
EMAIL_HOST_USER = os.getenv('EMAIL_HOST_USER', None)
EMAIL_HOST_PASSWORD = os.getenv('EMAIL_HOST_PASSWORD', None)
EMAIL_USE_TLS = os.getenv('EMAIL_USE_TLS', None)
EMAIL_USE_SSL = os.getenv('EMAIL_USE_SSL', None)
DEFAULT_FROM_EMAIL = os.getenv('DEFAULT_FROM_EMAIL', 'noreply@electeez.com')

if EMAIL_HOST:
    EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
else:
    EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'


REGISTRATION_OPEN = True     # django-registration
ACCOUNT_ACTIVATION_DAYS = 7  # django-registration
LOGIN_REDIRECT_URL = '/'     # django.contrib.auth
USE_X_FORWARDED_HOST = True  # reverse proxy support

# Password validation
# https://docs.djangoproject.com/en/1.9/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',  # noqa
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',  # noqa
    },
]


# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/

STATIC_URL = os.getenv('STATIC_URL', '/static/')
STATIC_ROOT = os.getenv(
    'STATIC_ROOT',
    Path(os.path.dirname(__file__)) / 'public'
)
if 'collectstatic' in sys.argv or not DEBUG:
    STATICFILES_STORAGE = 'django.contrib.staticfiles.storage.ManifestStaticFilesStorage'  # noqa

UWSGI_SPOOLER_MOUNT = os.getenv('UWSGI_SPOOLER_MOUNT')
UWSGI_SPOOLER_NAMES = os.getenv('UWSGI_SPOOLER_NAMES', '').split(',')
if UWSGI_SPOOLER_MOUNT and UWSGI_SPOOLER_NAMES:
    for name in UWSGI_SPOOLER_NAMES:
        path = os.path.join(UWSGI_SPOOLER_MOUNT, name)
        if not os.path.exists(path):
            os.makedirs(path)


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'formatters': {
        'simple': {
            'format': '%(levelname)s %(message)s'
        },
    },
    'loggers': {
        'django.request': {
            'handlers': ['console'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}



from django.apps import apps
from django.urls import reverse
from django.contrib.sites.models import Site
from crudlfap.html import *  # noqa

User = apps.get_model(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'))


@template('registration/login.html', App)
class LoginFormViewComponent(FormContainer):
    demo = False

    def context(self, *content, **context):
        context['view'].title = 'Login'
        return super().context(*content, **context)

    def to_html(self, *content, view, form, **kwargs):
        site = Site.objects.get_current(view.request)
        content = content or [
            H2('Welcome to ' + site.name, style='text-align: center;'),
        ]
        return super().to_html(
            Form(
                *content,
                # OAuthConnect(),
                # Span('Or enter email and password:', cls='center-text'),
                CSRFInput(view.request),
                form,
                Div(
                    MDCButtonRaised('Continue'),
                    A(
                        MDCButtonOutlined('forgot password'),
                        href=reverse('password_reset')
                    ),
                    style='display: flex; justify-content: space-between',
                ),
                method='POST',
                action=view.request.path_info,
                cls='form card',
            ),
            cls='',
        )


@template('registration/logged_out.html', App)
class LogoutViewComponent(FormContainer):
    def context(self, *content, **context):
        context['view'].title = 'Logout'
        return super().context(*content, **context)

    def __init__(self, *content, **context):
        super().__init__(
            H4('You have been logged out'),
            Section(
                'Thank you for spending time on our site today.',
            ),
            Div(
                MDCButton('Login again', tag='a', href=reverse('login')),
                style='display:flex; justify-content: flex-end;',
            ),
            cls='card',
            style='text-align: center',
        )


@template('registration/password_reset_form.html', App)
class PasswordResetCard(FormContainer):
    def to_html(self, *content, view, form, **context):
        return super().to_html(
            H4('Reset your password', style='text-align: center;'),
            Form(
                CSRFInput(view.request),
                form,
                MDCButtonRaised('Reset password'),
                method='POST',
                cls='form',
            ),
            cls='card',
        )


@template('registration/password_reset_confirm.html', App)
class PasswordResetConfirm(FormContainer):
    def context(self, *content, **context):
        context['view'].title = 'Password reset confirm'
        return super().context(*content, **context)

    def to_html(self, *content, view, form, **context):
        return super().to_html(
            H4('Reset your password', style='text-align: center;'),
            Form(
                CSRFInput(view.request),
                form,
                MDCButtonRaised('confirm'),
                method='POST',
                cls='form',
            ),
            cls='card',
        )


@template('registration/password_reset_complete.html', App)
class PasswordResetComplete(FormContainer):
    def __init__(self, **context):
        super().__init__(
            H4('Your password have been reset', cls='center-text'),
            Div(
                'You may go ahead and ',
                A('log in', href=reverse('login')),
                ' now',
            ),
        )


@template('registration/password_reset_done.html', App)
class PasswordResetDone(FormContainer):
    def __init__(self, **context):
        super().__init__(
            H4('A link has been sent to your email address'),
            A('Go to login page', href=reverse('login')),
            cls='card',
            style='text-align: center;',
        )



from django.conf import settings
from django.db import models
from django.utils.translation import gettext_lazy as _


class Song(models.Model):
    artist = models.ForeignKey('artist.artist', models.CASCADE)
    name = models.CharField(max_length=100, verbose_name=_('title'))
    duration = models.IntegerField(default=320)
    owner = models.ForeignKey(
        getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
        on_delete=models.CASCADE,
    )

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name



import operator
from functools import reduce

from django import forms
from django.db import models
from django.db.models import Q
from django.utils.translation import gettext as _


class SearchForm(forms.Form):
    q = forms.CharField(label=_('Search'), required=False)


class SearchMixin(object):
    def get_search_model(self):
        return self.model

    def get_search_fields(self):
        if hasattr(self.router, 'search_fields'):
            return self.router.search_fields
        return [
            f.name
            for f in self.search_model._meta.fields
            if isinstance(f, models.CharField)
        ]

    def get_search_form_class(self):
        if not self.search_fields:
            return

        return SearchForm

    def get_search_form(self):
        if self.search_fields:
            self.search_form = self.search_form_class(
                self.request.GET,
            )
            self.search_form.full_clean()
        else:
            self.search_form = None
        return self.search_form

    def search_filter(self, qs):
        q = self.search_form.cleaned_data.get('q', '')

        if not self.search_fields or not q:
            return qs

        return qs.filter(reduce(
            operator.or_,
            [
                Q(**{search_field + '__icontains': q})
                for search_field in self.search_fields
            ]
        )).distinct()



from django.conf import settings
from django.db import models
from django.utils import timezone


class PostQuerySet(models.QuerySet):
    def readable(self, user):
        if user.is_staff or user.is_superuser:
            return self

        published = models.Q(publish__lte=timezone.now())

        if not user.is_authenticated:
            return self.filter(published)

        return self.filter(models.Q(owner=user) | published)

    def editable(self, user):
        if not user.is_authenticated:
            return self.none()

        if user.is_staff or user.is_superuser:
            return self

        return self.filter(owner=user)


class PostManager(models.Manager):
    def get_queryset(self):
        return PostQuerySet(self.model, using=self._db)


class Post(models.Model):
    name = models.CharField(max_length=100, verbose_name='title')
    publish = models.DateTimeField(default=timezone.now)
    owner = models.ForeignKey(
        getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
        on_delete=models.CASCADE,
    )

    objects = PostManager()

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def editable(self, user):
        return user.is_staff or user.is_superuser or user.pk == self.owner_id



from django.utils.translation import gettext_lazy as _


class MenuMixin(object):
    def get_title_menu(self):
        """Return title for menu links to this view."""
        return _(self.view_label).capitalize()

    def get_menu(self):
        return None

    def get_menu_kwargs(self):
        return dict()

    def get_menu_views(self):
        views = []
        for view in self.router.views:
            for menu in view.menus:
                if menu in self.menus_display:
                    view = view.clone(
                        request=self.request,
                        **self.menu_kwargs,
                    )

                    if not view().has_perm():
                        continue
                    if view.urlname == self.urlname:
                        continue
                    if view.urlname in [v.urlname for v in views]:
                        continue
                    views.append(view)
        return views



from django.conf import settings
from django.contrib.sites.models import Site
from django.contrib import messages
from django.urls import reverse
from django.utils.translation import gettext as _
from ryzom_django_mdc.html import *  # noqa


class Html(Html):
    def to_html(self, *args, **kwargs):
        return '<!doctype html>' + super().to_html(*args, **kwargs)


UNPOLY_TARGET_ALL = '#main, .mdc-top-app-bar__title, #drawer .mdc-list'


class A(A):
    attrs = dict(
        up_target=UNPOLY_TARGET_ALL,
        # up_transition='cross-fade',
    )


class Form(Form):
    attrs = dict(
        up_target=A.attrs['up-target'],
        method='post',
    )


class Container(Div):
    sass = '''
    .Container
        display: flex
        flex-direction: column
        align-items: center
        max-width: 960px
        width: 96vw
        margin: auto

    .up-modal .Container
        width: 86vw

    @media screen and (max-width: 500px)
        .up-modal .up-modal-content
            padding: 10px
    '''


class FormContainer(Container):
    sass = '''
    .FormContainer
        max-width: 580px
        .mdc-text-field, .mdc-form-field, .mdc-select, form
            width: 100%
            max-width: 90vw
    '''


class PageMenu(Div):
    attrs = dict(cls='mdc-elevation--z2', style='margin-bottom: 10px')

    def __init__(self, *args, _next=None, _next_destructible=False, **kwargs):
        super().__init__(*args, **kwargs)
        self._next = _next
        # wether the next page is destructible, such as a detail view after a
        # delete post
        self._next_destructible = _next_destructible

    def link(self, view, context):
        button = A(
            MDCTextButton(
                view.label.capitalize(),
                icon=getattr(view, 'icon', None),
                tag='span',
                style={
                    'margin': '10px',
                    'color': getattr(view, 'color', 'inherit'),
                },
            ),
            href=view.url,
            style='text-decoration: none',
        )

        if getattr(view, 'controller', None) != 'modal':
            return button

        # wether going from the here to the next url is safe,
        # safe means that this menu view will not destroy the current url
        # which will be the case if we're in a detail view and making a
        # DELETE request on the object
        dangerous = (
            self._next_destructible
            and getattr(view, 'destructive', False)
        )

        button.attrs.up_layer = 'new'
        if dangerous:
            # in case the view will destroy the current view, enforce
            # list view
            next_url = str(view.router['list'].url)
            button.attrs.up_on_accepted = (
                f'up.visit("{view.router["list"].url}")'
            )
        else:
            next_url = self._next
            # we have to reload meanwhile unpoly 2 learns to reuse the
            # form response
            button.attrs.up_on_accepted = 'up.reload()'

        if '?' not in button.attrs['href']:
            button.attrs['href'] += '?'
        button.attrs['href'] += '&_next=' + next_url
        button.attrs.up_accept_location = next_url
        del button.attrs['up-target']
        return button

    def to_html(self, *content, **context):
        if 'page-menu' not in context:
            return super().to_html(*content, **context)

        content = list(content)
        content += [
            self.link(view, context)
            for view in context['page-menu']
            if view.urlname != context['view'].urlname
        ]

        return super().to_html(
            *content,
            '<div class="mdc-elevation-overlay"></div>',
            '<div class="mdc-button__ripple"></div>',
            **context,
        )


class Main(Main):
    pass


class Message(MDCSnackBar):
    icons = dict(
        debug='check_circle',
        info='check_circle',
        success='check_circle',
        warning='warning',
        error='error',
    )
    colors = dict(
        debug='green',
        info='yellow',
        success='green',
        warning='orange',
        error='red',
    )
    style = '''
        padding-left: 16px;
        display: flex;
        flex-direction: row;
        align-items: center
    '''


class Messages(Div):
    def to_html(self, *content, **context):
        msgs = messages.get_messages(context['view'].request)
        if not msgs:
            return ''

        return Div(*[
            Message(message)
            for message in msgs
        ]).to_html(*content, **context)


class Body(Body):
    style = 'margin: 0'
    sass = '''
    up-progress-bar
        background-color: var(--mdc-theme-secondary)
        height: 24px
        bottom: 0
        top: auto !important
        background: linear-gradient( \
            90deg, \
            var(--mdc-theme-primary), \
            var(--mdc-theme-secondary) \
        )
        width: 200vw
        animation: rotate 1s linear infinite
        position: absolute
    '''

    def __init__(self, *content, **attrs):
        self.drawer = mdcDrawer(id='drawer')
        self.bar = mdcTopAppBar()
        self.main_inner = Div(
            Messages(),
            *content,
            cls='main-inner',
        )
        self.main = Main(
            self.main_inner,
            cls='main mdc-drawer-app-content',
            id='main',
        )
        self.debug = settings.DEBUG
        super().__init__(
            self.bar,
            Div(
                self.drawer,
                self.main,
                cls='mdc-top-app-bar--fixed-adjust',
            ),
        )

    def py2js(self):
        up.compiler(
            '[data-mdc-auto-init]',
            lambda el: mdc.autoInit(el.parentElement)
        )
        if self.debug:
            up.log.enable()


class Spinner(Div):
    sass = '''
    .Spinner
        position: fixed
        float: right
        z-index: 20000
        cursor: pointer

        right: -24px
        top: 3px
        @media only screen and (max-width: 600px)
          top: -32px
          top: 0

        .lds-dual-ring
          display: inline-block
          width: 80px
          height: 80px

        .lds-dual-ring:after
          content: " "
          display: block
          width: 28px
          height: 28px
          margin: 8px
          border-radius: 50%
          border: 6px solid #fff
          border-color: var(--mdc-theme-primary) white var(--mdc-theme-primary) white
          animation: lds-dual-ring 1.2s linear infinite

        @keyframes lds-dual-ring
          0%
            transform: rotate(0deg)

          100%
            transform: rotate(360deg)
    '''  # noqa

    def py2js(self):
        up.proxy.config.slowDelay = 25
        up.compiler('.Spinner', lambda element: [
            up.on('up:request:late', lambda: up.element.show(element)),
            up.on('up:request:recover', lambda: up.element.hide(element)),
        ])

    def __init__(self):
        super().__init__(Div(cls='lds-dual-ring'), style='display:none')


class App(Html):
    body_class = Body
    scripts = [
        'https://unpkg.com/unpoly@2.6.1/unpoly.js',
        'https://unpkg.com/unpoly@2.6.1/unpoly-migrate.js',
    ]
    stylesheets = [
        'https://unpkg.com/unpoly@2.6.1/unpoly.min.css',
    ]
    sass = '''
    .up-modal .up-modal-viewport
        padding-left: 0
        padding-right: 0
        padding-top: 10px
    '''

    def to_html(self, *content, **context):
        if title := getattr(context['view'], 'title', None):
            self.head.content.append(Title(title))
        return super().to_html(*content, **context)


class NarrowCard(Div):
    style = {
        'max-width': '95vw',
        'margin': 'auto',
        'padding': '1em',
        'margin-top': '2em',
        'button': {
            'width': '100%',
        },
        '.MDCField label': {
            'width': '100%',
        }
    }


@template('crudlfap/form.html', App)
@template('crudlfap/update.html', App)
@template('crudlfap/create.html', App)
class FormTemplate(FormContainer):
    def to_html(self, view, form, **context):
        back = ''
        if next_ := view.request.GET.get('next', ''):
            back = A(
                MDCButton(
                    _('Back'),
                    tag='span',
                ),
                href=next_,
            )

        _next = view.request.GET.get(
            '_next',
            view.request.POST.get('_next', ''),
        )
        if _next:
            _next = Input(type='hidden', name='_next', value=_next)

        return super().to_html(
            H3(view.title),
            Form(
                form,
                CSRFInput(view.request),
                _next,
                back,
                MDCButtonRaised(getattr(view, 'title_submit', _('Submit'))),
                method='post',
                action=view.request.path_info,
            ),
        )


@template('crudlfap/api.html', Html)
class Swagger(Div):
    def py2js(self):
        el = document.getElementById('swagger-ui')
        ui = SwaggerUIBundle({
            url: el.attributes['schema-url'].value,
            dom_id: '#swagger-ui',
            deepLinking: true,
            presets: [
                SwaggerUIBundle.presets.apis,
                SwaggerUIStandalonePreset,
            ],
            plugins: [
                SwaggerUIBundle.plugins.DownloadUrl
            ],
            layout: "StandaloneLayout"
        })
        window.ui = ui

    def to_html(self, **context):
        registry = context['view'].registry
        return super().to_html(
            Script(src='/static/swagger-ui-bundle.js'),
            Script(src='/static/swagger-ui-standalone-preset.js'),
            Stylesheet(href='/static/swagger-ui.css'),
            Div(
                id='swagger-ui',
                schema_url=registry.views['schema'].url,
            ),
            **context
        )


@template('crudlfap/home.html', App, Container)
class Home(Div):
    def to_html(self, **context):
        site = Site.objects.get_current()
        user = context['view'].request.user
        return super().to_html(
            H1(
                f'Welcome to {site.name} {user.email}'
                if user.is_authenticated
                else
                f'Welcome to {site.name}!'
            ),
            MDCButtonRaised(
                'Login to continue',
                href=reverse('login'),
                up_target=UNPOLY_TARGET_ALL,
                tag='a',
            ) if not user.is_authenticated else '',
            Div('Then, navigate with the menu button at the north west'),
            **context
        )


@template('registration/logged_out.html', App, NarrowCard)
class LoggedOut(Div):
    def to_html(self, **context):
        return super().to_html(
            H1(_('Log out')),
            P(_('Thanks for spending some quality time with the Web site today.')),  # noqa
            A(
                _('Log in again'),
                href=reverse('login'),
            ),
            **context,
        )


@template('crudlfap/detail.html', App)
class ObjectDetail(Div):
    def to_html(self, *content, **context):
        table = MDCDataTable()
        table.thead.attrs.style.display = 'none'
        table.table.attrs.style.width = '100%'
        table.attrs.data_mdc_auto_init = False

        for field in context['view'].display_fields:
            table.tbody.addchild(MDCDataTableTr(
                MDCDataTableTh(field['field'].verbose_name.capitalize()),
                MDCDataTableTd(field['value']),
            ))

        return super().to_html(
            PageMenu(
                _next=context['view'].request.path,
                _next_destructible=True,
            ),
            *content,
            NarrowCard(table),
            **context
        )

    def context(self, *content, **context):
        context['page-menu'] = context['view'].router.get_menu(
            'object',
            context['view'].request,
            object=context['view'].object,
        )
        return super().context(*content, **context)


class ListAction(Div):
    style = {'display': 'inline-block'}

    def onclick(element):
        link = element.attributes.href.value + '?'
        for checkbox in document.querySelectorAll('[data-pk]:checked'):
            link += '&pk=' + checkbox.attributes['data-pk'].value
        up.modal.visit(link, {target: '.main-inner'})


class ListActions(Component):
    tag = 'list-actions'
    style = {'display': 'none'}

    class HTMLElement:
        def connectedCallback(self):
            this.previousElementSibling.addEventListener(
                'change',
                this.change.bind(this),
            )

            this.change({
                'target': this.previousElementSibling.querySelector(':checked')
            })

        def change(self, event):
            if event.target and event.target.checked:
                this.style.display = 'block'
            elif not this.previousElementSibling.querySelector(':checked'):
                this.style.display = 'none'


@template('crudlfap/list.html', App)
class ObjectList(Div):
    def context(self, *content, **context):
        if 'page-menu' not in context:
            context['page-menu'] = context['view'].router.get_menu(
                'model',
                context['view'].request,
            )
        return super().context(*content, **context)

    def to_html(self, *content, **context):
        checkbox = None
        context['listactions'] = context['view'].router.get_menu(
            'list_action',
            context['view'].request,
        )

        if context['listactions']:
            table_checkbox = MDCCheckboxInput()
            table_checkbox.attrs.addcls = 'mdc-data-table__header-row-checkbox'

            checkbox = MDCDataTableTh(
                table_checkbox,
                addcls='mdc-data-table__header-cell--checkbox',
            )
        thead = MDCDataTableThead(tr=MDCDataTableHeaderTr(checkbox))

        for column in context['view'].table.columns:
            thead.tr.addchild(self.th_component(column, **context))

        # align "actions" title to the right with the buttons
        thead.tr.content[-1].attrs.style['text-align'] = 'right'

        table = MDCDataTableResponsive(thead=thead, style={
            'min-width': '100%',
            'border-width': 0,
        })

        for row in context['view'].table.paginated_rows:
            table.tbody.addchild(
                self.row_component(row, **context)
            )

        if context['listactions']:
            table.addchild(self.listactions_component(**context))

        table.addchild(self.pagination_component(**context))

        return super().to_html(
            *content,
            self.drawer_component(**context) or '',
            PageMenu(
                _next=context['view'].request.path,
                _next_destructible=False,
            ),
            Div(
                self.search_component(**context) or '',
                table,
                cls='mdc-drawer__content',
                style='overflow-y: unset',
            ),
            **context,
        )

    def listactions_component(self, **context):
        return ListActions(
            *[
                ListAction(
                    MDCButton(
                        view.label,
                        icon=view.icon,
                        style='color: ' + getattr(view, 'color', 'inherit'),
                    ),
                    title=view.title,
                    href=view.url,
                    up_modal='.main-inner',
                    up_target=False,
                ) for view in context['view'].listactions
            ],
        )

    def search_component(self, **context):
        search_form = self.search_form_component(**context)

        filterset = getattr(context['view'], 'filterset', None)
        if not filterset:
            return search_form

        toggle = MDCDrawerToggle(
            Button(
                'filter_list',
                cls='material-icons mdc-icon-button',
                style='--mdc-ripple-fg-size:28px; --mdc-ripple-top:10px;',
            ),
            data_drawer_id='page-drawer',
        )

        filters_chips = Div(
            search=search_form or '',
            toggle=toggle if context['view'].filterset.form.fields else '',
            chips=Div(
                cls='mdc-chip-set',
                role='grid',
                style='display: inline-block',
            )
        )

        def remove_filter_url(name):
            get = context['view'].request.GET.copy()
            if name in get:
                del get[name]
            return context['view'].request.path_info + '?' + get.urlencode()

        for bf in filterset.form.visible_fields():
            value = context['view'].request.GET.get(bf.name, '')
            if not value:
                continue
            chip = MDCChip(
                Span(
                    role='button',
                    tabindex='0',
                    cls='mdc-chip__primary-action',
                    text=Span(
                        bf.label,
                        ': ',
                        str(bf.form.cleaned_data[bf.name]),
                        cls='mdc-chip__text',
                    ),
                ),
                icon=I(
                    'cancel',
                    cls=(
                        'material-icons',
                        'mdc-chip__icon',
                        'mdc-chip__icon--trailing',
                    ),
                    tabindex='-1',
                    role='button',
                ),
                tag='a',
                href=remove_filter_url(bf.name),
                up_target='main',
            )
            filters_chips.chips.addchild(chip)
        return filters_chips

    def drawer_component(self, **context):
        filterset = context['view'].filterset
        if not filterset.form.fields:
            return
        form = Form(
            method='get',
            action=context['view'].request.path_info,
            up_autosubmit=True,
            up_delay='200ms',
            up_target='.mdc-drawer__content',
        )
        for key in ('sort', 'q'):
            if key in context['view'].request.GET:
                form.addchild(Input(
                    name=key,
                    value=context['view'].request.GET[key],
                    type='hidden'
                ))
        for bf in filterset.form.visible_fields():
            filterfield = MDCFilterField(
                label=bf.label,
                widget=bf.to_component()
            )
            value = context['view'].request.GET.get(bf.name, '')
            if not value:
                filterfield.widget.style.display = 'none'
            form.addchild(filterfield)
        drawer = Aside(
            MDCDrawerToggle(
                MDCButton(_('Close'), icon='close'),
                data_drawer_id='page-drawer',
                style='text-align: right',
            ),
            Div(
                form,
                cls='mdc-drawer-app-content',
            ),
            id='page-drawer',
            cls='mdc-drawer mdc-drawer--dismissible',
            style='padding: 1em',
        )
        return drawer

    def search_form_component(self, **context):
        search_form = getattr(context['view'], 'search_form', None)

        if not search_form:
            return

        search_form = InlineForm(
            search_form,
            method='get',
            action=context['view'].request.path_info,
            up_autosubmit=True,
            up_delay='200ms',
            up_target='.mdc-data-table, .mdc-chip-set',
            style='margin-left: 14px',
        )
        for k, v in context['view'].request.GET.items():
            if k == 'q':
                continue
            search_form.addchild(Input(
                name=k,
                value=v,
                type='hidden',
            ))
        return search_form

    def row_component(self, row, **context):
        show_checkbox = False
        for listaction in context['listactions']:
            if listaction.clone(object=row.record)().has_perm():
                show_checkbox = True

        if show_checkbox:
            checkboxinput = MDCCheckboxInput(
                data_pk=str(row.record.pk)
            )
            checkboxinput.attrs.addcls = 'mdc-data-table__row-checkbox'
            tr = MDCDataTableTr(
                MDCDataTableTd(
                    checkboxinput,
                    addcls='mdc-data-table__cell--checkbox',
                )
            )
        elif context['listactions']:
            tr = MDCDataTableTr(MDCDataTableTd())
        else:
            tr = MDCDataTableTr()

        for column, cell in row.items():
            # todo: localize values
            tr.addchild(
                MDCDataTableTd(cell, data_label=column.header),
            )
            # todo: if is numeric
            # td.attrs.addcls = 'mdc-data-table__header-cell--numeric'
        return tr

    def th_component(self, column, **context):
        th = MDCDataTableTh(
            wrapper=Div(
                cls='mdc-data-table__header-cell-wrapper',
                label=Div(
                    cls='mdc-data-table__header-cell-label',
                    style='font-weight: 500',
                    text=Text(column.header),
                ),
            )
        )

        # sorting
        if column.orderable:
            th.attrs.addcls = 'mdc-data-table__header-cell--with-sort'
            if column.is_ordered:
                th.attrs.addcls = 'mdc-data-table__header-cell--sorted'
            get = context['view'].request.GET.copy()
            get['sort'] = column.order_by_alias.next
            href = ''.join([
                context['view'].request.path_info,
                '?',
                get.urlencode(),
            ])
            th.wrapper.content += [
                A(
                    cls=(
                        'mdc-icon-button',
                        'material-icons',
                        'mdc-data-table__sort-icon-button',
                    ),
                    aria_label='Sort by dessert',
                    aria_describedby='dessert-status-label',
                    up_target='table',
                    href=href,
                    text=Text(
                        'arrow_upward'
                        if column.order_by_alias.is_descending
                        else 'arrow_downward'
                    ),
                ),
                Div(
                    cls='mdc-data-table__sort-status-label',
                    aria_hidden='true',
                    id='dessert-status-label',
                ),
            ]
        return th

    def pagination_component(self, **context):
        def pageurl(n):
            get = context['view'].request.GET.copy()
            get['page'] = n
            return context['view'].request.path_info + '?' + get.urlencode()

        page = context['view'].table.page
        navigation = Div(
            Div(
                cls='mdc-data-table__pagination-total',
                text=Text(''.join([
                    str(page.start_index()),
                    '-',
                    str(page.paginator.per_page * page.number),
                    ' / ',
                    str(page.paginator.count),
                ]))
            ),
            A(
                cls=(
                    'mdc-icon-button',
                    'material-icons',
                    'mdc-data-table__pagination-button',
                ),
                disabled=page.number == 1,
                href=pageurl(1),
                icon=Div(cls='mdc-button__icon', text=Text('first_page')),
                up_target='.mdc-data-table',
            ),
            A(
                cls=(
                    'mdc-icon-button',
                    'material-icons',
                    'mdc-data-table__pagination-button',
                ),
                disabled=not page.has_previous(),
                icon=Div(cls='mdc-button__icon', text=Text('chevron_left')),
                href=pageurl(
                    page.number - 1
                    if page.has_previous()
                    else 1
                ),
                up_target='.mdc-data-table',
            ),
            A(
                cls=(
                    'mdc-icon-button',
                    'material-icons',
                    'mdc-data-table__pagination-button',
                ),
                disabled=not page.has_next(),
                icon=Div(cls='mdc-button__icon', text=Text('chevron_right')),
                href=pageurl(
                    page.number + 1
                    if page.has_next()
                    else page.paginator.num_pages
                ),
                up_target='.mdc-data-table',
            ),
            A(
                cls=(
                    'mdc-icon-button',
                    'material-icons',
                    'mdc-data-table__pagination-button',
                ),
                disabled=page.paginator.num_pages == page.number,
                icon=Div(cls='mdc-button__icon', text=Text('last_page')),
                href=pageurl(page.paginator.num_pages),
                up_target='.mdc-data-table',
            ),
            cls='mdc-data-table__pagination-navigation',
        )
        perpage = Div(
            Div(
                _('Rows per page'),
                cls='mdc-data-table__pagination-rows-per-page-label'
            ),
            select=MDCSelectPerPage(
                addcls=(
                    'mdc-select--outlined',
                    'mdc-select--no-label',
                    'mdc-data-table__pagination-rows-per-page-select',
                ),
                select=Select(*[
                    Option(
                        str(i),
                        value=i,
                        selected=page.paginator.per_page == i
                    )
                    for i in (3, 5, 7, 10, 25, 100)
                ])
            ),
            cls='mdc-data-table__pagination-rows-per-page',
        )
        return MDCDataTablePagination(
            perpage=perpage,
            navigation=navigation,
        )


class mdcTopAppBar(Header):
    def __init__(self, title='', buttons=None):
        self.title = title
        self.buttons = buttons or []
        super().__init__(
            Div(cls='mdc-top-app-bar__row'),
            cls='mdc-top-app-bar app-bar',
            id='app-bar',
            data_mdc_auto_init='MDCTopAppBar',
        )

    def to_html(self, view, **context):
        self.content[0].content = [Component(
            Section(
                Button(
                    'menu',
                    cls='material-icons mdc-top-app-bar__navigation-icon mdc-icon-button',  # noqa
                ),
                Span(
                    getattr(view, 'title', getattr(settings, 'SITE_NAME', '')),
                    cls='mdc-top-app-bar__title',
                ),
                cls='mdc-top-app-bar__section mdc-top-app-bar__section--align-start',  # noqa
            ),
            cls='mdc-top-app-bar__section mdc-top-app-bar__section--align-start',  # noqa
            tag='section',
        )]
        return super().to_html(**context)

    def nav():
        window.drawer.open = not window.drawer.open

    def close():
        if window.visualViewport.width < 800:
            window.drawer.open = False

    def py2js(self):
        up.compiler('#drawer, #app-bar, #main', self.setup)

    def setup():
        drawer = document.getElementById('drawer')
        app_bar = document.getElementById('app-bar')
        main = document.getElementById('main')

        window.drawer = mdc.drawer.MDCDrawer.attachTo(drawer)
        topAppBar = mdc.topAppBar.MDCTopAppBar.attachTo(app_bar)
        topAppBar.setScrollTarget(main)
        topAppBar.listen('MDCTopAppBar:nav', self.nav)

        for link in drawer.querySelectorAll('a'):
            link.addEventListener('click', self.close)


class mdcDrawer(Aside):
    menu_hooks = []

    def __init__(self, *content, **attrs):
        super().__init__(
            Div(
                *content,
                cls='mdc-drawer__content',
            ),
            cls='mdc-drawer mdc-drawer--dismissible mdc-top-app-bar--fixed-adjust',  # noqa
            data_mdc_auto_init='MDCDrawer',
            **attrs,
        )

    def to_html(self, *content, view, **context):
        request = view.request
        from .site import site

        menu_content = []
        if request.user.is_authenticated:
            menu_content.append(MDCListItem(str(request.user.email)))
        for view in site.get_menu('main', request):
            icon = getattr(view, 'icon', None)
            title = getattr(view, 'title', '')
            attrs = dict()
            if getattr(view, 'unpoly', True):
                attrs['up_target'] = UNPOLY_TARGET_ALL

            menu_content.append(
                MDCListItem(
                    title.capitalize(), icon=icon,
                    href=view.url,
                    style='text-decoration: none',
                    tag='a',
                    **attrs,
                ),
            )
        for hook in self.menu_hooks:
            menu_content = hook(request, menu_content)

        content = menu_content

        if request.session.get('become_user', None):
            content.append(
                A(
                    ' '.join([
                        str(_('Back to your account')),
                        request.session['become_user_realname'],
                    ]),
                    href=reverse('crudlfap:su'),
                    up_target=UNPOLY_TARGET_ALL,
                )
            )

        return super().to_html(MDCList(*content))


class mdcAppContent(Div):
    def __init__(self, *content):
        super().__init__(
            Component(
                *content,
                tag='main',
                cls='main-content',
                id='main-content',
            ),
            cls='mdc-drawer-app-content mdc-top-app-bar--fixed-adjust',
        )


class mdcSwitch(Component):
    def __init__(
        self, name, label=None, value=None, type=None, errors=None, help=None,
        required=False,
    ):
        super().__init__(
            Div(
                Div(cls='mdc-switch__track'),
                Div(
                    Div(cls='mdc-switch__thumb'),
                    Input(
                        type='checkbox',
                        id='id_' + name,
                        role='switch',
                        aria_checked='true' if bool(value) else '',
                        cls='mdc-switch__native-control',
                        checked=bool(value)
                    ),
                    cls='mdc-switch__thumb-underlay',
                ),
                cls='mdc-switch mdc-switch--checked',
                data_mdc_auto_init='MDCSwitch',
            ),
            Label(
                label or name.capitalize(),
                **{'for': 'id_' + name},
            )
        )



import json
import pytest

from crudlfap_auth.crudlfap import User


@pytest.mark.django_db
def test_api(client):
    user = User.objects.create(is_superuser=True, is_active=True)
    client.force_login(user)

    # test valid form
    response = client.post(
        '/artist/create',
        json.dumps(dict(
            name='test artist',
        )),
        content_type='application/json'
    )
    assert response.status_code == 201

    # test invalid form
    response = client.post(
        '/artist/create',
        json.dumps(dict(
            name='',
        )),
        content_type='application/json'
    )
    assert response.status_code == 405

    # test list get
    response = client.get(
        '/artist',
        HTTP_ACCEPT='application/json',
    )
    assert response.status_code == 200
    data = response.json()
    assert len(data['results']) == 1
    assert data['results'][0]['name'] == 'test artist'
    response = client.get(
        f'/artist/{data["results"][0]["id"]}',
        HTTP_ACCEPT='application/json'
    )
    assert response.json() == {'id': 1, 'name': 'test artist'}



import pytest
from django.contrib.auth.models import Group, Permission
from django.urls import resolve, reverse

from crudlfap import shortcuts as crudlfap

from .crudlfap import User


@pytest.fixture
def user():
    return User.objects.get_or_create(username='foo')[0]


@pytest.fixture
def suuser():
    return User.objects.create_superuser(
        username='suuser', password='123', email='')


@pytest.fixture
def group():
    return Group.objects.get_or_create(name='1')[0]


@pytest.mark.django_db
def test_get_menu(srf):
    srf.user = User.objects.create(is_superuser=True)
    result = crudlfap.site[User].get_menu('model', srf.get('/'))
    assert result[0].urlargs == []
    assert result[0].url == '/user/create'
    assert result[0].title == 'User: create'


def test_user_list_reverse():
    assert reverse('crudlfap:user:list') == '/user'


def test_user_create_reverse():
    assert reverse('crudlfap:user:create') == '/user/create'


def test_user_deleteobjects_reverse():
    assert reverse('crudlfap:user:deleteobjects') == '/user/deleteobjects'


def test_user_detail_reverse():
    assert reverse('crudlfap:user:detail', args=['a']) == '/user/a'


def test_user_delete_reverse():
    assert reverse('crudlfap:user:delete', args=['a']) == '/user/a/delete'


def test_user_update_reverse():
    assert reverse('crudlfap:user:update', args=['a']) == '/user/a/update'


def test_group_list_reverse():
    assert reverse('crudlfap:group:list') == '/group'


def test_group_create_reverse():
    assert reverse('crudlfap:group:create') == '/group/create'


def test_group_detail_reverse():
    assert reverse('crudlfap:group:detail', args=['1']) == '/group/1'


def test_group_delete_reverse():
    assert reverse('crudlfap:group:delete', args=['1']) == '/group/1/delete'


def test_group_update_reverse():
    assert reverse('crudlfap:group:update', args=['1']) == '/group/1/update'


def test_su_user_reverse():
    assert reverse('crudlfap:su') == '/su'


def test_url_reverse():
    assert reverse('crudlfap:url:list') == '/url'


@pytest.mark.django_db
def test_user_update_resolve(user):
    result = resolve('/user/foo/update')
    assert result.func.view_class.urlname == 'update'


@pytest.mark.django_db
def test_user_delete_resolve(user):
    result = resolve('/user/foo/delete')
    assert result.func.view_class.urlname == 'delete'


@pytest.mark.django_db
def test_user_deleteobjects_resolve(user):
    result = resolve('/user/deleteobjects')
    assert result.func.view_class.urlname == 'deleteobjects'


@pytest.mark.django_db
def test_user_create_resolve(user):
    result = resolve('/user/create')
    assert result.func.view_class.urlname == 'create'


@pytest.mark.django_db
def test_user_detail_resolve(user):
    result = resolve('/user/foo')
    assert result.func.view_class.urlname == 'detail'


def test_user_list_resolve():
    result = resolve('/user').func.view_class
    assert issubclass(result, crudlfap.ListView)
    assert result.__name__ == 'UserListView'
    assert result.urlname == 'list'
    assert result.url == '/user'


def test_group_list_resolve():
    result = resolve('/group').func.view_class
    assert issubclass(result, crudlfap.ListView)
    assert result.__name__ == 'GroupListView'
    assert result.urlname == 'list'
    assert result.url == '/group'


@pytest.mark.django_db
def test_group_update_resolve(group):
    result = resolve('/group/1/update')
    assert result.func.view_class.urlname == 'update'


@pytest.mark.django_db
def test_group_delete_resolve(group):
    result = resolve('/group/1/delete')
    assert result.func.view_class.urlname == 'delete'


@pytest.mark.django_db
def test_group_create_resolve(group):
    result = resolve('/group/create')
    assert result.func.view_class.urlname == 'create'


@pytest.mark.django_db
def test_group_detail_resolve(group):
    result = resolve('/group/1')
    assert result.func.view_class.urlname == 'detail'


def test_user_detail_url():
    view = crudlfap.site[User]['detail'](object=User(username='lol'))
    assert str(view.url) == '/user/lol'


def test_user_list_get(admin_client):
    result = admin_client.get('/user')
    assert result.status_code == 200


def test_user_create_get(admin_client):
    result = admin_client.get('/user/create')
    assert result.status_code == 200


@pytest.mark.django_db
def test_user_deleteobjects_get(admin_client):
    for i in range(100, 103):
        User.objects.get_or_create(pk=i, username='user{}'.format(i))
    result = admin_client.get('/user/deleteobjects?pks=101&pks=102')
    assert result.status_code == 200


def test_group_list_get(admin_client):
    result = admin_client.get('/group')
    assert result.status_code == 200


def test_group_create_get(admin_client):
    result = admin_client.get('/group/create')
    assert result.status_code == 200


@pytest.mark.django_db
def test_group_create_post(admin_client):
    result = admin_client.post('/group/create', dict(
        name='1',
    ))
    assert result.status_code == 302
    assert result['Location'] == '/group/1'


@pytest.mark.django_db
def test_group_create_post_with_permission(admin_client):
    permission = Permission.objects.filter().first()
    result = admin_client.post('/group/create', dict(
        name='1',
        permissions=permission.id
    ))
    assert result.status_code == 302
    assert result['Location'] == '/group/1'


@pytest.mark.django_db
def test_group_update_post(admin_client, group):
    permission = Permission.objects.filter().first()
    result = admin_client.post('/group/1/update', dict(
        name='1',
        permissions=permission.id
    ))
    assert result.status_code == 302
    assert result['Location'] == '/group/1'


@pytest.mark.django_db
def test_user_create_post(admin_client):
    result = admin_client.post('/user/create', dict(
        username='lol',
    ))
    assert result.status_code == 302
    assert result['Location'] == '/user/lol'


@pytest.mark.django_db
def test_user_create_post_with_next(admin_client):
    result = admin_client.post('/user/create', dict(
        username='lol',
        password='Download@123',
        _next='/user',
    ))
    assert result.status_code == 302
    assert result['Location'] == '/user'


@pytest.mark.django_db
def test_user_password_post(admin_client, user):
    result = admin_client.post('/user/foo/password', dict(
        old_password='Download@123',
        new_password1='Download123',
        new_password2='Download123',
    ))
    assert result.status_code == 302
    assert result['Location'] == '/user/foo'


@pytest.mark.django_db
def test_user_detail_get(client, user):
    result = client.get('/user/foo')
    assert result.status_code == 302


@pytest.mark.django_db
def test_user_detail_get_admin(admin_client, user):
    result = admin_client.get('/user/foo')
    assert result.status_code == 200


@pytest.mark.django_db
def test_user_password_get(admin_client, user):
    result = admin_client.get('/user/foo/password')
    assert result.status_code == 200
    assert b'id_new_password2' in result.content


@pytest.mark.django_db
def test_group_detail_get(admin_client, group):
    result = admin_client.get('/group/1')
    assert result.status_code == 200


@pytest.mark.django_db
def test_group_delete(admin_client):
    result = admin_client.get('/group/1/delete', dict(
        _next='/user',
    ))
    assert result.status_code == 404


def test_become_user(admin_client, admin_user, suuser, user):
    assert admin_client.session.get('become_user') is None

    result = admin_client.get('/user/suuser/su', follow=True)
    assert result.status_code == 200
    assert admin_client.session.get('become_user') == str(admin_user.id)

    result = admin_client.get('/su', follow=True)
    assert result.status_code == 200
    assert admin_client.session.get('become_user') is None

    result = admin_client.get('/user/foo/su', follow=True)
    assert result.status_code == 200
    assert admin_client.session.get('become_user') == str(admin_user.id)



import pytest
import re


@pytest.mark.django_db
def test_registration(client, mailoutbox):
    assert len(mailoutbox) == 0
    response = client.post('/registration/register/', dict(
        email='testregistration@example.com',
        username='testregistration',
        password1='!@aoe@#$',
        password2='!@aoe@#$',
    ))
    assert response.status_code == 302
    assert len(mailoutbox) == 1
    match = re.search('http://testserver[^\n]*', mailoutbox[0].body)
    assert match
    activate = match.group()
    response = client.get(activate)
    assert response.status_code == 302
    assert response['Location'] == '/registration/activate/complete/'
    response = client.post('/auth/login/', dict(
        username='testregistration',
        password='!@aoe@#$',
    ))
    assert response.status_code == 302
